import java.util.ArrayList;
import java.util.Iterator;

import javafx.animation.Animation;
import javafx.animation.KeyFrame;
import javafx.animation.Timeline;
import javafx.application.Application;
import javafx.geometry.Point2D;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.media.AudioClip;
import javafx.scene.paint.Color;
import javafx.scene.transform.Transform;
import javafx.stage.Stage;
import javafx.util.Duration;

/**
 * Basic scene graph demo
 * 
 * @author mike slattery
 * @version mar 2016
 */
public class Shooting extends Application {
	final String appName = "Shooting";
	final int FPS = 30; // frames per second
	final static int WIDTH = 600;
	final static int HEIGHT = 500;
	
	Scene theScene;
	Group root;
	Bullet ball;
	Blocker block;
	Gun gun;
	ArrayList<Target> targets = new ArrayList<Target>();
	
	AudioClip implode, crash;
	
	/**
	 * Set up initial data structures/values
	 */
	void initialize()
	{
		ball = null;
        block = new Blocker(340,60);
        gun = new Gun();
        gun.setTranslateX(100);
        gun.setTranslateY(250);
        
        for (int i = 0; i < 5; i++)
        	targets.add( new Target(500,50+i*80));
        
        implode = new AudioClip(ClassLoader.getSystemResource("exit.wav").toString());
        crash = new AudioClip(ClassLoader.getSystemResource("recycle.wav").toString());
		
	}
	
	void setHandlers(Scene scene)
	{
		scene.setOnKeyPressed(
				e -> fire()
				);
	}

	/**
	 *  Update variables for one time step
	 */
	public void update() {
		gun.update();
		if (ball != null) {
			ball.update();
			
			// Check targets
			Iterator<Target> iter = targets.iterator();
			while (iter.hasNext())
			{
				Target t = iter.next();
				if (t.getBoundsInParent().intersects(ball.getBoundsInParent())) {
					crash.play();
					iter.remove();  // remove target from ArrayList
					root.getChildren().remove(t);  // remove target from scene graph
				}
			}
			// Check blocking
			if (block.intersects(block.sceneToLocal(ball.getBoundsInParent()))) {
				implode.play();
				root.getChildren().remove(ball);
				ball = null;
			}
			else
			{
				ball = checkWalls(ball); //may reset ball to null
			}
		}

	}
	
	private Bullet checkWalls(Bullet b) {
        // die when outside of boundaries

        if (b.getTranslateX() > (theScene.getWidth() - 2*b.getRadius()) ||
                b.getTranslateX() < 0 ||
        	b.getTranslateY() > theScene.getHeight() - 2*b.getRadius() ||
                    b.getTranslateY() < 0){
            root.getChildren().remove(b);
            b = null;
        }
        return b;
    }
	
	
	// Launch a bubble
	void fire()
	{
		if (ball == null)
		{
			ball = new Bullet(8, Color.BLUE);
			Point2D loc = gun.localToScene(70, -8);
			ball.setTranslateX(loc.getX());
			ball.setTranslateY(loc.getY());
			
			Transform rot = gun.getTransforms().get(0);
			Point2D vel = rot.deltaTransform(4, 0);
			ball.vX = vel.getX(); ball.vY = vel.getY();
			ball.setVisible(true);
			root.getChildren().add(ball);
		}
	}
	
	/*
	 * Begin boiler-plate code...
	 * [Animation with initialization]
	 */
	public static void main(String[] args) {
		launch(args);
	}
	
	@Override
	public void start(Stage theStage) {
		theStage.setTitle(appName);

		root = new Group();
		theScene = new Scene(root, WIDTH, HEIGHT);
		theScene.setFill(Color.BLACK);
		theStage.setScene(theScene);

		// Initial setup
		initialize();
		root.getChildren().addAll(block, gun);
		root.getChildren().addAll(targets);
		
		setHandlers(theScene);
		
		// Setup and start animation loop (Timeline)
		KeyFrame kf = new KeyFrame(Duration.millis(1000 / FPS),
				e -> {
					// update position
					update();
				}
			);
		Timeline mainLoop = new Timeline(kf);
		mainLoop.setCycleCount(Animation.INDEFINITE);
		mainLoop.play();

		theStage.show();
	}
	/*
	 * ... End boiler-plate code
	 */
}
